package middleware

import (
	"context"

	pkgId "git.perx.ru/perxis/perxis-go/id"
	"git.perx.ru/perxis/perxis-go/pkg/organizations"

	"git.perx.ru/perxis/perxis-go/pkg/options"
	logzap "git.perx.ru/perxis/perxis-go/zap"
	"go.uber.org/zap"
)

type loggingMiddleware struct {
	logger *zap.Logger
	next   organizations.Organizations
}

func LoggingMiddleware(logger *zap.Logger) Middleware {
	return func(next organizations.Organizations) organizations.Organizations {
		return &loggingMiddleware{
			next:   next,
			logger: logger.With(logzap.Component("Organizations")),
		}
	}
}

func (m *loggingMiddleware) Create(ctx context.Context, org *organizations.Organization) (created *organizations.Organization, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(organizations.EventCreate),
	)

	created, err = m.next.Create(ctx, org)
	if err != nil {
		logger.Error("Failed to create", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog), logzap.Object(org))
		return
	}

	logger.Info("Organization created", logzap.Channels(logzap.Userlog), logzap.Object(created))

	return created, err
}

func (m *loggingMiddleware) Delete(ctx context.Context, id string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(organizations.EventDelete),
		logzap.Object(pkgId.NewOrganizationId(id)),
	)

	err = m.next.Delete(ctx, id)
	if err != nil {
		logger.Error("Failed to delete", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Organization deleted", logzap.Channels(logzap.Userlog))

	return err
}

func (m *loggingMiddleware) Find(ctx context.Context, filter *organizations.Filter, opts *options.FindOptions) (orgs []*organizations.Organization, total int, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
	)

	orgs, total, err = m.next.Find(ctx, filter, opts)
	if err != nil {
		logger.Error("Failed to find", zap.Error(err))
		return
	}

	return orgs, total, err
}

func (m *loggingMiddleware) Get(ctx context.Context, id string) (org *organizations.Organization, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Object(pkgId.NewOrganizationId(id)),
	)

	org, err = m.next.Get(ctx, id)
	if err != nil {
		logger.Error("Failed to get", zap.Error(err))
		return
	}

	return org, err
}

func (m *loggingMiddleware) Update(ctx context.Context, org *organizations.Organization) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(organizations.EventUpdate),
		logzap.Object(org),
	)

	err = m.next.Update(ctx, org)
	if err != nil {
		logger.Error("Failed to update", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Organization updated", logzap.Channels(logzap.Userlog))

	return err
}
